/*
   FILE:							       	
	linux_target.c							       
								       	
   DESCRIPTION:							       	
	Linux side of a sample application using the POSIX trace
	support. This Linux program also traces events into the
	active stream(s) that the RT-Linux side has created.

	See README file for details.
					       				
   Copyright (C) 2001
	Ana Garcia Fornes
	Andres Terrasa Barrena
	Ignacio Paches Giner
	Agustin Espinosa Minguet

  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.

  LAST REVISION:
        September 2001

*/

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

#include <trace.h>



//#define DEBUG



#define NTIMES 30


void timer_handler(int dummy) {
  return;
}


int main(void) {

  int               ntimes;
  trace_event_id_t  ev_linux;
  trace_event_id_t  ev_string;
  void              *data;
  char              s[32] = "Linux says: Hello world!";
  struct itimerval  timeout;
  struct sigaction  action;

  // Create a new type of event:
  posix_trace_eventid_open ("user event linux", &ev_linux); 

  // Register an event type which has been created in RT-Linux
  posix_trace_eventid_open ("user event string", &ev_string); 


  // Initialize the timeout value (oneshot, 500 ms):
  timeout.it_interval.tv_sec  = 0;
  timeout.it_interval.tv_usec = 0;
  timeout.it_value.tv_sec     = 0;
  timeout.it_value.tv_usec    = 500000;

  sigfillset(&(action.sa_mask));
  sigdelset(&(action.sa_mask), SIGALRM);
  action.sa_handler = timer_handler;
  sigaction(SIGALRM,&action,NULL);


  // Enter the main loop: trace this event and sleep for a timeout:
  for (ntimes = 0; ntimes < NTIMES; ntimes ++) {

#ifdef DEBUG
    fprintf(stderr, "linux_target: Tracing value %d\n", ntimes);
#endif

    data = (void *) & ntimes;
    posix_trace_event(ev_linux, data, sizeof(int));

    posix_trace_event(ev_string, s, sizeof(s));

    setitimer(ITIMER_REAL,&timeout,NULL);
    pause();
  }
    
  return 0;

}
