/*
   FILE:							       	
	trace/trace.h							       
								       	
   DESCRIPTION:							       	
	Private POSIX tracing definitions 
					       				
   Copyright (C) 2001,2002
	Ana Garcia Fornes
	Andres Terrasa Barrena
	Ignacio Paches Giner
	Agustin Espinosa Minguet

  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.

  LAST REVISION:
        April 2002

*/

#ifndef __TRACE_POSIX_TRACE_H
#define __TRACE_POSIX_TRACE_H


// Import which POSIX tracing options are currently supported:
#include "trace_options.h"


//#ifdef _POSIX_TRACE


#ifdef MODULE
#include <linux/time.h>
#else
#include <sys/time.h>
#endif



/*
  NOTE:
        This header file is mostly ordered and labelled according to the 
	POSIX "P1003.1q: Tracing" standard document.
*/


/* 
   2.8.2. Minimum values 
*/
#define _POSIX_TRACE_SYS_MAX             _POSIX_TRACE_STREAMS
#define _POSIX_TRACE_USER_EVENT_MAX     32
#define _POSIX_TRACE_NAME_MAX           64
#define _POSIX_TRACE_EVENT_NAME_MAX     30




/* 
   24.2. Data definitions 
*/

// 24.2.1.2. posix_trace_event_info needs this:
typedef  int         _trace_event_id_t;


// 24.2.2 Trace stream attributes
typedef struct {
  int     magic;                 // Magic number

  char    generation_version[_POSIX_TRACE_NAME_MAX];
                                 //  Origin and version of the trace system
                                
  char    trace_name[_POSIX_TRACE_NAME_MAX];
                                 //  Identification of the trace stream
  struct timespec 
          creation_time;         //  Stream creation time.
  struct timespec 
          clock_resolution;      //  Timestamps clock resolution
  size_t  stream_min_size;       //  User-defined stream minimum size      
  size_t  max_data_size;         //  User-defined event maximum data size
  size_t  data_size;             //  Real maximum data size of any event;
  size_t  header_size;           //  Size of the event header (to make Linux know)
  int     stream_full_policy;    //  Policy when the stream becomes full
#ifdef _POSIX_TRACE_INHERITANCE
  int     inheritance;           //  Inheritance policy
#endif
#ifdef _POSIX_TRACE_TOLOG
  size_t  log_max_size;          //  Maximum size (bytes) of the stream's log
  int     log_full_policy;       //  Policy when the log becomes full
#endif
} _trace_attr_t;






/*
  Implementation-dependent data definitions
*/

typedef  int            _trace_id_t;


#define  _POSIX_TRACE_SYSTEM_EVENT_MAX  32
 
#define  _POSIX_TRACE_EVENT_MAX         (_POSIX_TRACE_USER_EVENT_MAX + \
					 _POSIX_TRACE_SYSTEM_EVENT_MAX)

typedef char           bool;


typedef struct  {
  int  user_set;
  int  system_set;
} _trace_event_set_t;



// Each type of event (user and system) has this structure
typedef struct event_type_info_struct {
  _trace_id_t        eventid;   // eventid < 0 means the event isn't used.
  char               name[_POSIX_TRACE_EVENT_NAME_MAX];             
} event_type_info_t; 


// The set of stream events is implemented as an array of this type
typedef struct stream_event_info_struct {
  _trace_event_id_t             event_id;
  void                        * prog_address;
  int                           truncation_status;
  pthread_t                     thread_id;  
  hrtime_t                      timestamp;
  int                           overflow;
  size_t                        data_length;
  void                        * user_data;
} stream_event_t;


/*
   Macro exported to the different functions which trace an event.
*/

#define posix_trace_get_invoking_address(calling_address) \
           calling_address = ((int) __builtin_return_address(0)) - 5;



//#endif // #ifdef _POSIX_TRACE

#endif // #ifndef __TRACE_POSIX_TRACE_H
