/*
 * Copyright (c) 2004 Andres Terrasa.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Further, this software is distributed without any warranty that it is
 * free of the rightful claim of any third person regarding infringement
 * or the like.  Any license provided herein, whether implied or
 * otherwise, applies only to this software file.  Patent licenses, if
 * any, provided herein do not apply to combinations of this program with
 * other software, or any other product whatsoever.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information: Andres Terrasa UPV
 *
 * mail: aterrasa@dsic.upv.es
 *
 */
/**********************************************************
 * 
 *    POSIX Trace TEST
 * 
 *    TEST IDENTIFIER	: ltp_ptrace_perf1.c 
 * 
 *    EXECUTED BY	: root 
 * 
 *    TEST TITLE	:  POSIX Trace performance test 1
 * 
 *    TEST CASE TOTAL	: 1
 * 
 *    CPU TYPES		: i386 
 * 
 *    AUTHOR		: Andres Terrasa
 * 
 *    CO-PILOT		: Agustin Espinosa
 * 
 *    DATE STARTED	: 14/10/04
 * 
 *    INITIAL RELEASE	: RTL-PTM 1.0
 * 
 *    TEST CASES
 * 
 * 	1.-  Temporal performance of the POSIX Tracing
 *	
 *    INPUT SPECIFICATIONS
 *      None
 * 
 *    OUTPUT SPECIFICATIONS
 *      None
 * 	
 *    DURATION
 * 	Terminates 
 * 
 *    SIGNALS
 *      No signals used
 *
 *    RESOURCES
 * 	RTLinux scheduler module with POSIX Tracing support enabled
 * 
 *    ENVIRONMENTAL NEEDS
 *      OCERA_DIR must be set to the ocera root directory and exported.
 *      LTPROOT must be set to the LTP root directory and exported.
 * 
 *    SPECIAL PROCEDURAL REQUIREMENTS
 * 	You need root privileges to run this test
 * 
 *    INTERCASE DEPENDENCIES
 *
 * 
 *    DETAILED DESCRIPTION
 *      This is a test for the POSIX Trace support in RT-Linux
 *
 *	This test measures of the costs of tracing and retrieving
 *	events using the POSIX Tracing API.
 *
 *	Costs are calculated by using the function rdtsc, which 
 *	measures the time in CPU clock ticks. Measurements are
 *	stored in a mbuff shared memory, from where a Linux program
 *	retrieves them and store them in some files. These files
 *	are later analyzed in order to compute some basic
 *	statistics (minimum, maximum, average and variance values).
 *
 *	The test module has three entry parameters:
 *
 *	- period : period of the tasks tracing and retrieving the
 *	           events for measurement.
 *
 *	- data_size: Size in bytes of the data being traced and 
 *	           retrieved along with each event. Valid values
 *		   are between 0 and 4096 bytes.
 *
 *       - policy: full policy to be used by the stream.
 *
 *	This makes easier to apply the performance test in different
 *	scenarios, although currently this test is applied to one
 *      configuration only.
 *
 * 	Setup:
 * 	  Check if the test has root privileges
 * 	  Remove the RTLinux modules, if loaded.
 * 
 * 	Test:
 * 	  1.- Load the required RTLinux modules
 *        2.- Load the test module to store measurements to a mbuff
 *        3.- Run the program to get the measurements from the mbuff
 *        4.- Run the basic statistics program to the measurements
 *
 * 	Cleanup:
 * 	  Remove the test module
 * 	  Print the results and exit
 *
 * 
 *#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#**/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <test.h>
#include <time.h>
#include <linux/module.h>
#include <sys/klog.h>
#include <string.h>

extern int query_module(const char *name, int which,
			void *buf, size_t bufsize, size_t *ret);


/*
  LTP global variables and definitions 
*/

char *TCID = "ptrace_test_perf1";/* Test program identifier. */
int TST_TOTAL = 1;		/* Total number of test cases. */
extern int Tst_count;		/* Test Case counter for tst_* routines */


/*
  Test variables and definitions
*/


#define MODULE_PERIOD    1000000  /* 1 ms */
#define MODULE_DATASIZE  4        /* 4 bytes */
#define MODULE_POLICY    1        /* 1 = LOOP, 2 = UNTIL_FULL */
#define MODULE_DELAY     25



#define UNLOAD_RTLINUX "/sbin/rmmod rtl_sched mbuff rtl_fifo rtl_posixio \
                       rtl_time rtl >/dev/null 2>&1"

#define LOAD_TESTMOD   "/sbin/insmod \
                        $LTPROOT/testcases/bin/test_ptrace_perf1_module.o \
                        period=%ld data_size=%d policy=%d >/dev/null 2>&1"

#define UNLOAD_TESTMOD "/sbin/rmmod test_ptrace_perf1_module >/dev/null 2>&1"

#define RUN_WRITER     "$LTPROOT/testcases/bin/test_ptrace_perf1_writer \
                        perf1_measurements %f"

#define RUN_STATISTIC  "$LTPROOT/testcases/bin/test_ptrace_perf1_statistic \
                        %s \"%s\""

#define RUN_MHZ         "grep MHz /proc/cpuinfo | cut -f 2 -d: | head -1 \
                         > perf1_mhz_tmp"

#define MSG_LOAD_FAIL   "Failed to load test_ptrace_stress2_module"
#define MSG_WRITER_FAIL "Failed to run the writer program"
#define MSG_STAT_FAIL   "Failed to run the statistics program"
#define MSG_UNLOAD_FAIL "Failed to unload test_ptrace_stress2_module"
#define MSG_MHZ_FAIL    "Failed to get CPU Mhz"

#define MSG_TESTING1   "Starting phase 1/2: loading performance test module..."
#define MSG_TESTING2    "Starting phase 2/2: running statistics program..."
//#define MSG_OK1         "Test 1/2 passed: measurements successfully generated"
#define MSG_OK2         "Phase 2/2 passed: measurements successfully analyzed"
#define MSG_SUCCESS     "Performance test passsed"


char filenames[11][64] = { 
  "--line",
  "--header",
  "--line",
  "perf1_measurements.tr.times",
  "perf1_measurements.tr1.times",
  "perf1_measurements.tr2.times",
  "--line",
  "perf1_measurements.gn.times",
  "perf1_measurements.gn1.times",
  "perf1_measurements.gn2.times",
  "--line",
};

char descriptions[11][64] = { 
  "",
  "",
  "",
  "Trace (all values)  ",
  "Trace (1st values)  ",
  "Trace (2nd values)  ",
  "",
  "GetNext (all values)",
  "GetNext (1st values)",
  "GetNext (2nd values)",
  "",
};


void cleanup ()
{
    system (UNLOAD_TESTMOD);
    system (UNLOAD_RTLINUX);
    system ("rm -f $LTPROOT/testcases/bin/perf1_measurements.*");
    system ("rm -f $LTPROOT/testcases/bin/perf1_mhz_tmp");
    tst_exit ();
}



void setup ()
{

    /* must run test as root */
    if (geteuid () != 0) {
	tst_brkm (TBROK, tst_exit, "You must be root to run this test");
    }

    /* If the test module is inserted, remove it first */
    tst_resm (TINFO, "Setup: unload RT-Linux and previous test modules...");
    system (UNLOAD_TESTMOD);
    system (UNLOAD_RTLINUX);

    if (query_module ("rtl", 0, NULL, 0, NULL) == 0) {
	tst_brkm (TBROK, tst_exit,
		  "RT-Linux modules already loaded: Setup failed");
    }

    /* load RTLinux modules */
    if (system ("/sbin/insmod rtl quiet=1 >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load rtl module");
    }
    if (system ("/sbin/insmod rtl_time >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load rtl_time module");
    }
    if (system ("/sbin/insmod rtl_posixio >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load rtl_posixio module");
    }
    if (system ("/sbin/insmod rtl_fifo >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load rtl_fifo module");
    }
    if (system ("/sbin/insmod mbuff >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load mbuff module");
    }
    if (system ("/sbin/insmod rtl_sched >/dev/null 2>&1") != 0) {
	tst_brkm (TBROK, cleanup, "Unable to load rtl_sched module");
    }

    system("dmesg -c >/dev/null 2>&1");
}




/* load the module to be tested */
void load_module (long period, int data_size, int policy)
{
    char load_testmod[256];

    sprintf (load_testmod, LOAD_TESTMOD, period, data_size, policy);
    if (system (load_testmod) != 0) {
	tst_resm (TINFO, MSG_LOAD_FAIL);
    }

}



/* unload the module to be tested */
void unload_module ()
{
    if (system (UNLOAD_TESTMOD) != 0) {
	tst_brkm (TFAIL, cleanup, MSG_UNLOAD_FAIL);
    }
}


/* run program that gets the mbuff contents and writes them to files */
void run_writer ()
{
    char run_wlinux[512];
    FILE *mhz_file;
    float mhz;

    if (system(RUN_MHZ) != 0) {
	tst_brkm (TFAIL, cleanup, MSG_MHZ_FAIL);
    }

    if ((mhz_file=fopen("perf1_mhz_tmp", "r")) == NULL) {
	tst_brkm (TFAIL, cleanup, MSG_MHZ_FAIL);
    }
    fscanf(mhz_file, " %f\n", &mhz);
    fclose(mhz_file);

    sprintf (run_wlinux, RUN_WRITER, mhz);
    if (system (run_wlinux) != 0) {
	tst_brkm (TFAIL, cleanup, MSG_WRITER_FAIL);

    }
}




/* run program that processes the statistics of the measurements */
void run_statistics ()
{
    char run_statistic[512];
    int i;

    for (i = 0; i < 11; i++) {

	sprintf (run_statistic, RUN_STATISTIC, filenames[i],
		 descriptions[i]);

	if (system (run_statistic) != 0) {
	    tst_brkm (TFAIL, cleanup, MSG_STAT_FAIL);
	}
    }
}




/* capture and process the kernel module output (printk) */
char *process_module_output ()
{
    char dmesg[16 * 1024];
    char *result_line;

    klogctl (3, dmesg, 16392);
    result_line = strstr (dmesg, TCID);
    strtok (result_line, ":");
    if (strncmp (strtok (NULL, ":"), " PASS ", 6)) {
      tst_brkm (TFAIL, cleanup, strtok (NULL, "\n"));
      tst_exit ();
    }
    return strtok(NULL, "\n");
}




/* main test function */
int main (int ac, char **av, char **env)
{
    struct timespec delay, dummy;
    char *module_result_ok;

    Tst_count = 0;

    setup ();
    tst_resm (TINFO, MSG_TESTING1);
    load_module (MODULE_PERIOD, MODULE_DATASIZE, MODULE_POLICY);

    tst_resm (TINFO, "The module must run for 25 seconds. Waiting....");
    delay.tv_sec = MODULE_DELAY;
    delay.tv_nsec = 0;
    nanosleep (&delay, &dummy);

    module_result_ok = process_module_output();
    run_writer ();
    unload_module ();
    tst_resm (TINFO, module_result_ok);

    tst_resm (TINFO, MSG_TESTING2);
    run_statistics ();
    tst_resm (TINFO, MSG_OK2);

    tst_resm (TPASS, MSG_SUCCESS);
    cleanup ();
    return 0;
}
