/*

   FILE:

	test_ptrace_perf1_statistic.c
								
   DESCRIPTION: 
        Program that given a list of numbers (execution times) from a
	file, computes and prints out the miminum, maximum, average 
	and variance of these values

	Use:
	test_ptrace_perf1_statistic --header
	test_ptrace_perf1_statistic --line
	test_ptrace_perf1_statistic <filename> "description"

	Entry parameters:

	--header: prints out a header (compatible with the printing
	          of statistic values)
	--line :  prints out a line 
        
	<filename>: file containing the measurements (one per line)

	"description" : Textual description of the measurements
	            (upto 20 characters)
	

   Copyright (C) 2003,2004
	Andres Terrasa Barrena
	Ana Garcia Fornes
	Agustin Espinosa Minguet


  This is free software;  you can  redistribute it  and/or modify it under
  terms of the  GNU General Public License as published  by the Free Soft-
  ware  Foundation;  either version 2,  or (at your option) any later ver-
  sion.  It is distributed in the hope that it will be useful, but WITH-
  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for  more details.  You should have  received  a copy of the GNU General
  Public License;  see file COPYING.  If not, write to  the Free Software 
  Foundation,  59 Temple Place - Suite 330,  Boston, MA 02111-1307, USA.


  LAST REVISION:
        October 2004
	(based on the previous "test_ptrace_perf1_statistic" (c) A. Terrasa, Dec. 2003)
*/

#include <stdio.h> 
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>


#define MAX_DESCRIPTION 21

int main (int argc, char *argv[]) {


  int      error;
  int      nsamples;
  int      min, max;
  double   total, avg, variance;
  double   sd, square_diffs;
  int      value;
  FILE    *fin;
  char     description[MAX_DESCRIPTION];
  int      desc_available = 0;

  //  inputfile = stdin;
  //outputfile = stdout;

  if (argc < 2) {
    fprintf(stderr, "Use: test_ptrace_perf1_statistic --header\n"); 
    fprintf(stderr, "Use: test_ptrace_perf1_statistic --line\n"); 
    fprintf(stderr, "Use: test_ptrace_perf1_statistic inputfile [description]\n"); 
    exit(1);
  };

  if (! strcmp("--header", argv[1])){
    fprintf(stdout,  "Value measured        Samples     Min      Max     Avg        Variance\n");
    exit(0);
  }

  if (! strcmp("--line", argv[1])){
    fprintf(stdout,  "----------------------------------------------------------------------\n");
    exit(0);
  }




  if (argc > 2) {
    strncpy(description, argv[2], MAX_DESCRIPTION);
    desc_available = 1;
    //    fprintf(stderr, "test_ptrace_perf1_statistic: %s\n", description);
  }


  if ((fin = fopen(argv[1], "r")) == NULL) {
        fprintf(stderr, "test_ptrace_perf1_statistic: Error: cannot open input file %s\n",
		argv[1]); 
	exit (1);
  }  
  

  // First walk through the input file: mininum, maximum and average:
  if ((error = fscanf(fin, "%d\n", &value)) <= 0) {
    fprintf(stderr, "test_ptrace_perf1_statistic: Error: input file doesn't contain valid data\n");
    exit (1);
  }


   
  max = min = value;
  total = value;
  avg   = 0.0;
  variance = 0.0;
  nsamples = 1;

  while (! feof(fin)) {
  
    fscanf(fin, "%d\n", &value);

    total += value;
    nsamples ++;

    if (value > max) {
      max = value;
    }

    if (value < min) {
      min = value;
    }
  }
   
  avg = (double) (total / (double) nsamples);


  // Second walk through the input file: square differences and variance:
  rewind(fin);
  square_diffs = 0;

  fscanf(fin, "%d\n", &value);
  sd = ((double) value) - avg;
  sd *= sd;
  square_diffs += sd;

   while (! feof(fin)) {
  
    fscanf(fin, "%d\n", &value);
    sd = ((double) value) - avg;
    sd *= sd;
    square_diffs += sd;
   }
 
   variance = square_diffs / (double) nsamples;
   // Present the results:
   fprintf(stdout, "%s: ", desc_available? description : argv[1]);
   fprintf(stdout, "%6d %8d %8d %10.04f %12.04f\n",
	   nsamples, min, max, avg, variance);
   

  /*
    fprintf(stderr, "\n");
    fprintf(stderr, "            <row>\n");
    fprintf(stderr, "            <entry><para>%s</para></entry>\n", argv[1]);
    fprintf(stderr, "            <entry><para>%.04f</para></entry>\n", (double) min);
    fprintf(stderr, "            <entry><para>%.04f</para></entry>\n", (double) max);
    fprintf(stderr, "            <entry><para>%.04f</para></entry>\n", (double) avg);
    fprintf(stderr, "            <entry><para>%.04f</para></entry>\n", (double) variance);
    fprintf(stderr, "            </row>\n\n");
  */


  return 0;

}
